"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const curl_1 = require("./curl");
const credentials_1 = require("./credentials");
const args_1 = require("./args");
const isUrl = require('is-url');
const aws4 = require('aws4');
const minimist = require('minimist');
function version() {
    console.log(require('../package.json').version);
}
function help() {
    console.log(`
curl, but with AWS Signature Version 4

usage: aws4curl [aws4curl options] [curl options]

aws4curl options:
  --aws-region AWS Region to sign requests with (required)
  --aws-service AWS Service (required)

AWS Credentials:
Environment variables AWS_ACCESS_KEY and AWS_SECRET_KEY must be defined.
If AWS_SESSION_TOKEN is available, it will be used to generate the header
"X-Amz-Security-Token"

curl options:
  Every flag and argument will be passed to your installed curl
`);
}
function removeArg(argv, arg, numberFollowing) {
    const index = argv.indexOf(arg);
    if (index !== -1) {
        return argv.splice(index, numberFollowing);
    }
}
function addHeader(to, headers, header) {
    if (headers[header]) {
        to.push('-H');
        to.push(`${header}: ${headers[header]}`);
    }
}
function run() {
    let { argv } = process;
    // Remove initial two args (node and file)
    argv.splice(0, 2);
    const parsedArgs = minimist(argv);
    if (parsedArgs.version) {
        return version();
    }
    if (parsedArgs.help) {
        return help();
    }
    credentials_1.validateCredentials();
    // Get AWS Region
    const region = parsedArgs['aws-region'];
    if (typeof region !== 'string') {
        throw new Error('--aws-region is mandatory (e.g. eu-central-1)');
    }
    removeArg(argv, '--aws-region', 2);
    // Get AWS Service
    const service = parsedArgs['aws-service'];
    if (typeof service !== 'string') {
        throw new Error('--aws-service is mandatory (e.g. execute-api)');
    }
    removeArg(argv, '--aws-service', 2);
    // Get URL
    const url = parsedArgs._[0];
    if (!isUrl(url)) {
        throw new Error('An URL is required for curl to work');
    }
    const parsedUrl = url_1.parse(url);
    const { host, path } = parsedUrl;
    if (!host) {
        throw new Error('URL does not have a valid host');
    }
    if (!path) {
        throw new Error('URL does not have a valid path');
    }
    // Handle curl headers
    const headers = args_1.getHeaders(parsedArgs);
    // Remove all headers, we will add them back later after signing them
    while (removeArg(argv, '-H', 2)) { }
    while (removeArg(argv, '--header', 2)) { }
    const body = args_1.getBody(parsedArgs);
    const method = args_1.getMethod(parsedArgs);
    const options = {
        host,
        path,
        region,
        service,
        headers,
        body,
        method
    };
    // Finally, sign the request, which mutates the variable
    aws4.sign(options);
    if (!options.headers) {
        throw new Error('The signature could not be created');
    }
    // Build curl arguments
    const curlArgs = [];
    Object.keys(options.headers).forEach(headerKey => {
        addHeader(curlArgs, options.headers, headerKey);
    });
    curlArgs.push(...argv);
    curl_1.curl(curlArgs);
}
exports.run = run;
